@@ -16,6 +16,7 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 # Copyright (C) 2012 Lanedo GmbH
+# Copyright (C) 2012-2017 Aleksander Morgado <aleksander@aleksander.es>
 #
 
 import string
@@ -38,6 +39,7 @@ class Message:
         self.name = dictionary['name']
         # The specific message ID
         self.id = dictionary['id']
+
         # The type, which must always be 'Message' or 'Indication'
         self.type = dictionary['type']
         # The version info, optional
@@ -45,6 +47,16 @@ class Message:
         self.static = True if 'scope' in dictionary and dictionary['scope'] == 'library-only' else False
         self.abort = True if 'abort' in dictionary and dictionary['abort'] == 'yes' else False
 
+        # libqmi version where the message was introduced
+        self.since = dictionary['since'] if 'since' in dictionary else None
+        if self.since is None:
+            raise ValueError('Message ' + self.name + ' requires a "since" tag specifying the major version where it was introduced')
+
+        # The vendor id if this command is vendor specific
+        self.vendor = dictionary['vendor'] if 'vendor' in dictionary else None
+        if self.type == 'Indication' and self.vendor is not None:
+            raise ValueError('Vendor-specific indications unsupported')
+
         # The message prefix
         self.prefix = 'Qmi ' + self.type
 
@@ -64,7 +76,8 @@ class Message:
                                 'Output',
                                 dictionary['output'] if 'output' in dictionary else None,
                                 common_objects_dictionary,
-                                self.static)
+                                self.static,
+                                self.since)
 
         self.input = None
         if self.type == 'Message':
@@ -76,7 +89,8 @@ class Message:
                                    'Input',
                                    dictionary['input'] if 'input' in dictionary else None,
                                    common_objects_dictionary,
-                                   self.static)
+                                   self.static,
+                                   self.since)
 
 
     """
