@@ -18,7 +18,7 @@
  * Boston, MA 02110-1301 USA.
  *
  * Copyright (C) 2012 Google, Inc.
- * Copyright (C) 2012-2015 Aleksander Morgado <aleksander@aleksander.es>
+ * Copyright (C) 2012-2017 Aleksander Morgado <aleksander@aleksander.es>
  */
 
 #ifndef _LIBQMI_GLIB_H_
@@ -33,6 +33,7 @@
 #include "qmi-client.h"
 #include "qmi-proxy.h"
 #include "qmi-message.h"
+#include "qmi-message-context.h"
 #include "qmi-enums.h"
 #include "qmi-utils.h"
 
@@ -55,6 +56,9 @@
 #include "qmi-enums-pds.h"
 #include "qmi-pds.h"
 
+#include "qmi-enums-pdc.h"
+#include "qmi-pdc.h"
+
 #include "qmi-enums-pbm.h"
 #include "qmi-pbm.h"
 
