@@ -18,6 +18,7 @@
  * Boston, MA 02110-1301 USA.
  *
  * Copyright (C) 2012 Google Inc.
+ * Copyright (C) 2012-2017 Aleksander Morgado <aleksander@aleksander.es>
  */
 
 #ifndef _LIBQMI_GLIB_QMI_ENUMS_NAS_H_
@@ -51,6 +52,8 @@
  * @QMI_NAS_RADIO_INTERFACE_TD_SCDMA: TD-SCDMA.
  *
  * Radio interface technology.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_RADIO_INTERFACE_UNKNOWN     = -1,
@@ -64,6 +67,12 @@ typedef enum {
     QMI_NAS_RADIO_INTERFACE_TD_SCDMA    = 0x09
 } QmiNasRadioInterface;
 
+/**
+ * qmi_nas_radio_interface_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasActiveBand:
  * @QMI_NAS_ACTIVE_BAND_BC_0: Band class 0.
@@ -146,6 +155,8 @@ typedef enum {
  * @QMI_NAS_ACTIVE_BAND_TDSCDMA_F: TD-SCDMA Band F.
  *
  * Band classes.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_ACTIVE_BAND_BC_0 = 0,
@@ -228,6 +239,12 @@ typedef enum {
     QMI_NAS_ACTIVE_BAND_TDSCDMA_F = 205
 } QmiNasActiveBand;
 
+/**
+ * qmi_nas_active_band_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasNetworkServiceDomain:
  * @QMI_NAS_NETWORK_SERVICE_DOMAIN_NONE: No service.
@@ -237,6 +254,8 @@ typedef enum {
  * @QMI_NAS_NETWORK_SERVICE_DOMAIN_UNKNOWN: Unknown service.
  *
  * Network Service Domain.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_NETWORK_SERVICE_DOMAIN_NONE    = 0x00,
@@ -246,6 +265,12 @@ typedef enum {
     QMI_NAS_NETWORK_SERVICE_DOMAIN_UNKNOWN = 0x04,
 } QmiNasNetworkServiceDomain;
 
+/**
+ * qmi_nas_network_service_domain_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasEvdoSinrLevel:
  * @QMI_NAS_EVDO_SINR_LEVEL_0: -9 dB.
@@ -259,6 +284,8 @@ typedef enum {
  * @QMI_NAS_EVDO_SINR_LEVEL_8: +9 dB.
  *
  * EV-DO SINR level.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_EVDO_SINR_LEVEL_0 = 0,
@@ -272,6 +299,12 @@ typedef enum {
     QMI_NAS_EVDO_SINR_LEVEL_8 = 8
 } QmiNasEvdoSinrLevel;
 
+/**
+ * qmi_nas_evdo_sinr_level_get_string:
+ *
+ * Since: 1.0
+ */
+
 /*****************************************************************************/
 /* Helper enums for the 'QMI NAS Get Signal Strength' request/response */
 
@@ -288,6 +321,8 @@ typedef enum {
  * @QMI_NAS_SIGNAL_STRENGTH_REQUEST_LTE_RSRP: Request LTE RSRP information.
  *
  * Extra information to request when gathering Signal Strength.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_SIGNAL_STRENGTH_REQUEST_NONE       = 0,
@@ -301,6 +336,12 @@ typedef enum {
     QMI_NAS_SIGNAL_STRENGTH_REQUEST_LTE_RSRP   = 1 << 7
 } QmiNasSignalStrengthRequest;
 
+/**
+ * qmi_nas_signal_strength_request_build_string_from_mask:
+ *
+ * Since: 1.0
+ */
+
 /*****************************************************************************/
 /* Helper enums for the 'QMI NAS Network Scan' request/response */
 
@@ -312,6 +353,8 @@ typedef enum {
  * @QMI_NAS_NETWORK_SCAN_TYPE_TD_SCDMA: TD-SCDMA network.
  *
  * Flags to use when specifying which networks to scan.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_NETWORK_SCAN_TYPE_GSM      = 1 << 0,
@@ -320,6 +363,12 @@ typedef enum {
     QMI_NAS_NETWORK_SCAN_TYPE_TD_SCDMA = 1 << 3
 } QmiNasNetworkScanType;
 
+/**
+ * qmi_nas_network_scan_type_build_string_from_mask:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasNetworkStatus:
  * @QMI_NAS_NETWORK_STATUS_CURRENT_SERVING: Network is in use, current serving.
@@ -332,6 +381,8 @@ typedef enum {
  * @QMI_NAS_NETWORK_STATUS_NOT_PREFERRED: Network is not preferred.
  *
  * Flags to specify the status of a given network.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_NETWORK_STATUS_CURRENT_SERVING = 1 << 0,
@@ -344,6 +395,12 @@ typedef enum {
     QMI_NAS_NETWORK_STATUS_NOT_PREFERRED   = 1 << 7
 } QmiNasNetworkStatus;
 
+/**
+ * qmi_nas_network_status_build_string_from_mask:
+ *
+ * Since: 1.0
+ */
+
 /*****************************************************************************/
 /* Helper enums for the 'QMI NAS Initiate Network Register' request/response */
 
@@ -353,12 +410,20 @@ typedef enum {
  * @QMI_NAS_NETWORK_REGISTER_TYPE_MANUAL: Manual network registration.
  *
  * Type of network registration.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_NETWORK_REGISTER_TYPE_AUTOMATIC = 0x01,
     QMI_NAS_NETWORK_REGISTER_TYPE_MANUAL    = 0x02
 } QmiNasNetworkRegisterType;
 
+/**
+ * qmi_nas_network_register_type_get_string:
+ *
+ * Since: 1.0
+ */
+
 /*****************************************************************************/
 /* Helper enums for the 'QMI NAS Get Serving System' request/response */
 
@@ -371,6 +436,8 @@ typedef enum {
  * @QMI_NAS_REGISTRATION_STATE_UNKNOWN: Unknown.
  *
  * Status of the network registration.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_REGISTRATION_STATE_NOT_REGISTERED           = 0x00,
@@ -380,6 +447,12 @@ typedef enum {
     QMI_NAS_REGISTRATION_STATE_UNKNOWN                  = 0x04
 } QmiNasRegistrationState;
 
+/**
+ * qmi_nas_registration_state_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasAttachState:
  * @QMI_NAS_ATTACH_STATE_UNKNOWN: Unknown attach state.
@@ -387,6 +460,8 @@ typedef enum {
  * @QMI_NAS_ATTACH_STATE_DETACHED: Detached.
  *
  * Domain attach state.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_ATTACH_STATE_UNKNOWN  = 0x00,
@@ -394,6 +469,12 @@ typedef enum {
     QMI_NAS_ATTACH_STATE_DETACHED = 0x02,
 } QmiNasAttachState;
 
+/**
+ * qmi_nas_attach_state_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasNetworkType:
  * @QMI_NAS_NETWORK_TYPE_UNKNOWN: Unknown.
@@ -401,6 +482,8 @@ typedef enum {
  * @QMI_NAS_NETWORK_TYPE_3GPP: 3GPP network.
  *
  * Type of network.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_NETWORK_TYPE_UNKNOWN = 0x00,
@@ -408,12 +491,20 @@ typedef enum {
     QMI_NAS_NETWORK_TYPE_3GPP    = 0x02,
 } QmiNasNetworkType;
 
+/**
+ * qmi_nas_network_type_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasRoamingIndicatorStatus:
  * @QMI_NAS_ROAMING_INDICATOR_STATUS_ON: Roaming.
  * @QMI_NAS_ROAMING_INDICATOR_STATUS_OFF: Home.
  *
  * Status of the roaming indication.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_ROAMING_INDICATOR_STATUS_ON  = 0x00,
@@ -421,6 +512,12 @@ typedef enum {
     /* next values only for 3GPP2 */
 } QmiNasRoamingIndicatorStatus;
 
+/**
+ * qmi_nas_roaming_indicator_status_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasDataCapability:
  * @QMI_NAS_DATA_CAPABILITY_NONE: None or unknown.
@@ -439,6 +536,8 @@ typedef enum {
  * @QMI_NAS_DATA_CAPABILITY_DC_HSDPA_PLUS: DC-HSDPA+.
  *
  * Data capability of the network.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_DATA_CAPABILITY_NONE          = 0x00,
@@ -457,6 +556,12 @@ typedef enum {
     QMI_NAS_DATA_CAPABILITY_DC_HSDPA_PLUS = 0x0D
 } QmiNasDataCapability;
 
+/**
+ * qmi_nas_data_capability_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasServiceStatus:
  * @QMI_NAS_SERVICE_STATUS_NONE: No service.
@@ -466,6 +571,8 @@ typedef enum {
  * @QMI_NAS_SERVICE_STATUS_POWER_SAVE: Device in power save mode.
  *
  * Status of the service.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_SERVICE_STATUS_NONE             = 0x00,
@@ -475,6 +582,12 @@ typedef enum {
     QMI_NAS_SERVICE_STATUS_POWER_SAVE       = 0x04
 } QmiNasServiceStatus;
 
+/**
+ * qmi_nas_service_status_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasHdrPersonality:
  * @QMI_NAS_HDR_PERSONALITY_UNKNOWN: Unknown.
@@ -482,6 +595,8 @@ typedef enum {
  * @QMI_NAS_HDR_PERSONALITY_EHRPD: eHRPD.
  *
  * HDR personality type.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_HDR_PERSONALITY_UNKNOWN = 0x00,
@@ -489,6 +604,12 @@ typedef enum {
     QMI_NAS_HDR_PERSONALITY_EHRPD   = 0x02,
 } QmiNasHdrPersonality;
 
+/**
+ * qmi_nas_hdr_personality_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasCallBarringStatus:
  * @QMI_NAS_CALL_BARRING_STATUS_NORMAL_ONLY: Normal calls only.
@@ -498,6 +619,8 @@ typedef enum {
  * @QMI_NAS_CALL_BARRING_STATUS_UNKNOWN: Unknown.
  *
  * Status of the call barring functionality.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_CALL_BARRING_STATUS_NORMAL_ONLY    = 0x00,
@@ -507,6 +630,12 @@ typedef enum {
     QMI_NAS_CALL_BARRING_STATUS_UNKNOWN        = -1
 } QmiNasCallBarringStatus;
 
+/**
+ * qmi_nas_call_barring_status_get_string:
+ *
+ * Since: 1.0
+ */
+
 /*****************************************************************************/
 /* Helper enums for the 'QMI NAS Get Home Network' request/response */
 
@@ -517,6 +646,8 @@ typedef enum {
  * @QMI_NAS_NETWORK_DESCRIPTION_DISPLAY_UNKNOWN: Unknown.
  *
  * Setup to define whether the network description should be displayed.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_NETWORK_DESCRIPTION_DISPLAY_NO      = 0x00,
@@ -524,6 +655,12 @@ typedef enum {
     QMI_NAS_NETWORK_DESCRIPTION_DISPLAY_UNKNOWN = 0xFF
 } QmiNasNetworkDescriptionDisplay;
 
+/**
+ * qmi_nas_network_description_display_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasNetworkDescriptionEncoding:
  * @QMI_NAS_NETWORK_DESCRIPTION_ENCODING_UNSPECIFIED: Unspecified.
@@ -532,6 +669,8 @@ typedef enum {
  * @QMI_NAS_NETWORK_DESCRIPTION_ENCODING_GSM: GSM 7-bit.
  *
  * Type of encoding used in the network description.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_NETWORK_DESCRIPTION_ENCODING_UNSPECIFIED = 0x00,
@@ -540,6 +679,12 @@ typedef enum {
     QMI_NAS_NETWORK_DESCRIPTION_ENCODING_GSM         = 0x09
 } QmiNasNetworkDescriptionEncoding;
 
+/**
+ * qmi_nas_network_description_encoding_get_string:
+ *
+ * Since: 1.0
+ */
+
 /*****************************************************************************/
 /* Helper enums for the 'QMI NAS Get Technology Preference' request/response */
 
@@ -554,6 +699,8 @@ typedef enum {
  * @QMI_NAS_RADIO_TECHNOLOGY_PREFERENCE_LTE: LTE.
  *
  * Flags to specify the radio technology preference.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_RADIO_TECHNOLOGY_PREFERENCE_AUTO          = 0,
@@ -565,6 +712,12 @@ typedef enum {
     QMI_NAS_RADIO_TECHNOLOGY_PREFERENCE_LTE           = 1 << 5
 } QmiNasRadioTechnologyPreference;
 
+/**
+ * qmi_nas_radio_technology_preference_build_string_from_mask:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasPreferenceDuration:
  * @QMI_NAS_PREFERENCE_DURATION_PERMANENT: Permanent.
@@ -576,6 +729,8 @@ typedef enum {
  * @QMI_NAS_PREFERENCE_DURATION_INTERNAL_ONE_CALL_3: Internal reason 3, one call.
  *
  * Duration of the preference setting.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_PREFERENCE_DURATION_PERMANENT           = 0x00,
@@ -587,6 +742,12 @@ typedef enum {
     QMI_NAS_PREFERENCE_DURATION_INTERNAL_ONE_CALL_3 = 0x06
 } QmiNasPreferenceDuration;
 
+/**
+ * qmi_nas_preference_duration_get_string:
+ *
+ * Since: 1.0
+ */
+
 /*****************************************************************************/
 /* Helper enums for the 'QMI NAS Get/Set System Selection Preference'
  * requests/responses */
@@ -601,6 +762,8 @@ typedef enum {
  * @QMI_NAS_RAT_MODE_PREFERENCE_TD_SCDMA: TD-SCDMA.
  *
  * Flags specifying radio access technology mode preference.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_RAT_MODE_PREFERENCE_CDMA_1X     = 1 << 0,
@@ -611,6 +774,12 @@ typedef enum {
     QMI_NAS_RAT_MODE_PREFERENCE_TD_SCDMA    = 1 << 5
 } QmiNasRatModePreference;
 
+/**
+ * qmi_nas_rat_mode_preference_build_string_from_mask:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasCdmaPrlPreference:
  * @QMI_NAS_CDMA_PRL_PREFERENCE_A_SIDE_ONLY: System A only.
@@ -618,6 +787,8 @@ typedef enum {
  * @QMI_NAS_CDMA_PRL_PREFERENCE_ANY: Any system.
  *
  * Flags specifying the preference when using CDMA Band Class 0.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_CDMA_PRL_PREFERENCE_A_SIDE_ONLY = 0x0001,
@@ -625,6 +796,12 @@ typedef enum {
     QMI_NAS_CDMA_PRL_PREFERENCE_ANY         = 0x3FFF
 } QmiNasCdmaPrlPreference;
 
+/**
+ * qmi_nas_cdma_prl_preference_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasRoamingPreference:
  * @QMI_NAS_ROAMING_PREFERENCE_OFF: Only non-roaming networks.
@@ -633,6 +810,8 @@ typedef enum {
  * @QMI_NAS_ROAMING_PREFERENCE_ANY: Don't filter by roaming when acquiring networks.
  *
  * Roaming preference.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_ROAMING_PREFERENCE_OFF          = 0x01,
@@ -641,30 +820,52 @@ typedef enum {
     QMI_NAS_ROAMING_PREFERENCE_ANY          = 0xFF
 } QmiNasRoamingPreference;
 
+/**
+ * qmi_nas_roaming_preference_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasNetworkSelectionPreference:
  * @QMI_NAS_NETWORK_SELECTION_PREFERENCE_AUTOMATIC: Automatic.
  * @QMI_NAS_NETWORK_SELECTION_PREFERENCE_MANUAL: Manual.
  *
  * Network selection preference.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_NETWORK_SELECTION_PREFERENCE_AUTOMATIC = 0x00,
     QMI_NAS_NETWORK_SELECTION_PREFERENCE_MANUAL    = 0x01
 } QmiNasNetworkSelectionPreference;
 
+/**
+ * qmi_nas_network_selection_preference_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasChangeDuration:
  * @QMI_NAS_CHANGE_DURATION_PERMANENT: Permanent.
  * @QMI_NAS_CHANGE_DURATION_POWER_CYCLE: Until the next power cycle.
  *
  * Duration of the change setting.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_CHANGE_DURATION_POWER_CYCLE = 0x00,
     QMI_NAS_CHANGE_DURATION_PERMANENT   = 0x01
 } QmiNasChangeDuration;
 
+/**
+ * qmi_nas_change_duration_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasServiceDomainPreference:
  * @QMI_NAS_SERVICE_DOMAIN_PREFERENCE_CS_ONLY: Circuit-switched only.
@@ -674,6 +875,8 @@ typedef enum {
  * @QMI_NAS_SERVICE_DOMAIN_PREFERENCE_PS_DETACH:Packet-switched dettach.
  *
  * Service domain preference.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_SERVICE_DOMAIN_PREFERENCE_CS_ONLY   = 0x00,
@@ -683,6 +886,12 @@ typedef enum {
     QMI_NAS_SERVICE_DOMAIN_PREFERENCE_PS_DETACH = 0x04,
 } QmiNasServiceDomainPreference;
 
+/**
+ * qmi_nas_service_domain_preference_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasGsmWcdmaAcquisitionOrderPreference:
  * @QMI_NAS_GSM_WCDMA_ACQUISITION_ORDER_PREFERENCE_AUTOMATIC: Automatic.
@@ -690,6 +899,8 @@ typedef enum {
  * @QMI_NAS_GSM_WCDMA_ACQUISITION_ORDER_PREFERENCE_WCDMA: WCDMA first, then GSM.
  *
  * GSM/WCDMA acquisition order preference.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_GSM_WCDMA_ACQUISITION_ORDER_PREFERENCE_AUTOMATIC = 0x00,
@@ -697,6 +908,12 @@ typedef enum {
     QMI_NAS_GSM_WCDMA_ACQUISITION_ORDER_PREFERENCE_WCDMA     = 0x02
 } QmiNasGsmWcdmaAcquisitionOrderPreference;
 
+/**
+ * qmi_nas_gsm_wcdma_acquisition_order_preference_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasTdScdmaBandPreference:
  * @QMI_NAS_TD_SCDMA_BAND_PREFERENCE_A: Band A.
@@ -707,6 +924,8 @@ typedef enum {
  * @QMI_NAS_TD_SCDMA_BAND_PREFERENCE_F: Band F.
  *
  * Flags to specify TD-SCDMA-specific frequency band preferences.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_TD_SCDMA_BAND_PREFERENCE_A = 1 << 0,
@@ -717,6 +936,12 @@ typedef enum {
     QMI_NAS_TD_SCDMA_BAND_PREFERENCE_F = 1 << 5
 } QmiNasTdScdmaBandPreference;
 
+/**
+ * qmi_nas_td_scdma_band_preference_build_string_from_mask:
+ *
+ * Since: 1.0
+ */
+
 /*****************************************************************************/
 /* Helper enums for the 'QMI NAS Get System Info' request/response */
 
@@ -735,7 +960,11 @@ typedef enum {
  * @QMI_NAS_ROAMING_STATUS_PARTIAL_SERVICE: Partial service.
  * @QMI_NAS_ROAMING_STATUS_BANNER_ON: Banner on.
  * @QMI_NAS_ROAMING_STATUS_BANNER_OFF: Banner off.
-*/
+ *
+ * Roaming status.
+ *
+ * Since: 1.0
+ */
 typedef enum {
     QMI_NAS_ROAMING_STATUS_OFF                 = 0x00,
     QMI_NAS_ROAMING_STATUS_ON                  = 0x01,
@@ -753,6 +982,12 @@ typedef enum {
     QMI_NAS_ROAMING_STATUS_BANNER_OFF          = 0x0C
 } QmiNasRoamingStatus;
 
+/**
+ * qmi_nas_roaming_status_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasHdrProtocolRevision:
  * @QMI_NAS_HDR_PROTOCOL_REVISION_NONE: None.
@@ -761,6 +996,8 @@ typedef enum {
  * @QMI_NAS_HDR_PROTOCOL_REVISION_REL_B: HDR Rel B.
  *
  * HDR protocol revision.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_HDR_PROTOCOL_REVISION_NONE  = 0x00,
@@ -769,6 +1006,12 @@ typedef enum {
     QMI_NAS_HDR_PROTOCOL_REVISION_REL_B = 0x03
 } QmiNasHdrProtocolRevision;
 
+/**
+ * qmi_nas_hdr_protocol_revision_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasWcdmaHsService:
  * @QMI_NAS_WCDMA_HS_SERVICE_HSDPA_HSUPA_UNSUPPORTED: HSDPA and HSUPA not supported.
@@ -779,7 +1022,10 @@ typedef enum {
  * @QMI_NAS_WCDMA_HS_SERVICE_HSDPA_PLUS_HSUPA_SUPPORTED: HSDPA+ and HSUPA supported.
  * @QMI_NAS_WCDMA_HS_SERVICE_DC_HSDPA_PLUS_SUPPORTED: DC-HSDPA+ supported.
  * @QMI_NAS_WCDMA_HS_SERVICE_DC_HSDPA_PLUS_HSUPA_SUPPORTED: DC-HSDPA+ and HSUPA supported.
+ *
  * Call status on high speed.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_WCDMA_HS_SERVICE_HSDPA_HSUPA_UNSUPPORTED       = 0x00,
@@ -792,6 +1038,12 @@ typedef enum {
     QMI_NAS_WCDMA_HS_SERVICE_DC_HSDPA_PLUS_HSUPA_SUPPORTED = 0x07
 } QmiNasWcdmaHsService;
 
+/**
+ * qmi_nas_wcdma_hs_service_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasCellBroadcastCapability:
  * @QMI_NAS_CELL_BROADCAST_CAPABILITY_UNKNOWN: Unknown.
@@ -799,6 +1051,8 @@ typedef enum {
  * @QMI_NAS_CELL_BROADCAST_CAPABILITY_ON: Cell broadcast supported.
  *
  * Cell broadcast support.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_CELL_BROADCAST_CAPABILITY_UNKNOWN = 0x00,
@@ -806,6 +1060,12 @@ typedef enum {
     QMI_NAS_CELL_BROADCAST_CAPABILITY_ON      = 0x02
 } QmiNasCellBroadcastCapability;
 
+/**
+ * qmi_nas_cell_broadcast_capability_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasSimRejectState:
  * @QMI_NAS_SIM_REJECT_STATE_SIM_UNAVAILABLE: SIM not available.
@@ -815,6 +1075,8 @@ typedef enum {
  * @QMI_NAS_SIM_REJECT_STATE_SIM_CS_PS_INVALID: SIM invalid for circuit-switched and packet-switched connections.
  *
  * Reject information of the SIM.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_SIM_REJECT_STATE_SIM_UNAVAILABLE   = 0,
@@ -824,18 +1086,32 @@ typedef enum {
     QMI_NAS_SIM_REJECT_STATE_SIM_CS_PS_INVALID = 4
 } QmiNasSimRejectState;
 
+/**
+ * qmi_nas_sim_reject_state_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasCdmaPilotType:
  * @QMI_NAS_CDMA_PILOT_TYPE_ACTIVE: the pilot is part of the active set.
  * @QMI_NAS_CDMA_PILOT_TYPE_NEIGHBOR: the pilot is part of the neighbor set.
  *
  * The pilot set the pilot belongs to.
+ *
+ * Since: 1.0
  */
 typedef enum {
     QMI_NAS_CDMA_PILOT_TYPE_ACTIVE   = 0,
     QMI_NAS_CDMA_PILOT_TYPE_NEIGHBOR = 1,
 } QmiNasCdmaPilotType;
 
+/**
+ * qmi_nas_cdma_pilot_type_get_string:
+ *
+ * Since: 1.0
+ */
+
 /**
  * QmiNasDayOfWeek:
  * @QMI_NAS_DAY_OF_WEEK_MONDAY: Monday
@@ -847,6 +1123,8 @@ typedef enum {
  * @QMI_NAS_DAY_OF_WEEK_SUNDAY: Sunday
  *
  * The day of the week.
+ *
+ * Since: 1.4
  */
 typedef enum {
     QMI_NAS_DAY_OF_WEEK_MONDAY    = 0,
@@ -858,6 +1136,12 @@ typedef enum {
     QMI_NAS_DAY_OF_WEEK_SUNDAY    = 6
 } QmiNasDayOfWeek;
 
+/**
+ * qmi_nas_day_of_week_get_string:
+ *
+ * Since: 1.4
+ */
+
 /**
  * QmiNasDaylightSavingsAdjustment:
  * @QMI_NAS_DAYLIGHT_SAVINGS_ADJUSTMENT_NONE: no adjustment
@@ -865,6 +1149,8 @@ typedef enum {
  * @QMI_NAS_DAYLIGHT_SAVINGS_ADJUSTMENT_TWO_HOURS: two hours adjustment
  *
  * The number of hours a time is adjusted for daylight savings.
+ *
+ * Since: 1.4
  */
 typedef enum {
     QMI_NAS_DAYLIGHT_SAVINGS_ADJUSTMENT_NONE      = 0,
@@ -872,6 +1158,12 @@ typedef enum {
     QMI_NAS_DAYLIGHT_SAVINGS_ADJUSTMENT_TWO_HOURS = 2
 } QmiNasDaylightSavingsAdjustment;
 
+/**
+ * qmi_nas_daylight_savings_adjustment_get_string:
+ *
+ * Since: 1.4
+ */
+
 /*****************************************************************************/
 /* Helper enums for the 'QMI NAS Get Cell Location Info' request/response */
 
@@ -884,6 +1176,8 @@ typedef enum {
  * @QMI_NAS_WCDMA_RRC_STATE_CELL_DCH: WCDMA RRC state is CELL_DCH.
  *
  * RRC state.
+ *
+ * Since: 1.10
  */
 typedef enum {
     QMI_NAS_WCDMA_RRC_STATE_DISCONNECTED = 0,
@@ -893,8 +1187,15 @@ typedef enum {
     QMI_NAS_WCDMA_RRC_STATE_CELL_DCH     = 4
 } QmiNasWcdmaRrcState;
 
+/**
+ * qmi_nas_wcdma_rrc_state_get_string:
+ *
+ * Since: 1.10
+ */
+
 /*****************************************************************************/
 /* Helper enums for the 'QMI NAS Get LTE Cphy CA Info' request/response */
+
 /**
  * QmiNasDLBandwidth:
  * @QMI_NAS_DL_BANDWIDTH_1_4: 1.4 MHz
@@ -907,18 +1208,26 @@ typedef enum {
  * @QMI_NAS_DL_BANDWIDTH_UNKNOWN: Unknown
  *
  * DL Bandwidth.
+ *
+ * Since: 1.16
  */
 typedef enum {
-    QMI_NAS_DL_BANDWIDTH_1_4  = 0,
-    QMI_NAS_DL_BANDWIDTH_3,
-    QMI_NAS_DL_BANDWIDTH_5,
-    QMI_NAS_DL_BANDWIDTH_10,
-    QMI_NAS_DL_BANDWIDTH_15,
-    QMI_NAS_DL_BANDWIDTH_20,
-    QMI_NAS_DL_BANDWIDTH_INVALID,
+    QMI_NAS_DL_BANDWIDTH_1_4      = 0,
+    QMI_NAS_DL_BANDWIDTH_3        = 1,
+    QMI_NAS_DL_BANDWIDTH_5        = 2,
+    QMI_NAS_DL_BANDWIDTH_10       = 3,
+    QMI_NAS_DL_BANDWIDTH_15       = 4,
+    QMI_NAS_DL_BANDWIDTH_20       = 5,
+    QMI_NAS_DL_BANDWIDTH_INVALID  = 6,
     QMI_NAS_DL_BANDWIDTH_UNKNOWN  = 0xFF
 } QmiNasDLBandwidth;
 
+/**
+ * qmi_nas_dl_bandwidth_get_string:
+ *
+ * Since: 1.16
+ */
+
 /**
  * QmiNasScellState:
  * @QMI_NAS_SCELL_STATE_DECONFIGURED: Deconfigured
@@ -926,11 +1235,101 @@ typedef enum {
  * @QMI_NAS_SCELL_STATE_ACTIVATED: Activated
  *
  * SCell State.
+ *
+ * Since: 1.16
  */
 typedef enum {
     QMI_NAS_SCELL_STATE_DECONFIGURED = 0,
-    QMI_NAS_SCELL_STATE_DEACTIVATED,
-    QMI_NAS_SCELL_STATE_ACTIVATED
+    QMI_NAS_SCELL_STATE_DEACTIVATED  = 1,
+    QMI_NAS_SCELL_STATE_ACTIVATED    = 2
 } QmiNasScellState;
 
+/**
+ * qmi_nas_scell_state_get_string:
+ *
+ * Since: 1.16
+ */
+
+/*****************************************************************************/
+/* Helper enums for the 'QMI NAS Get Operator Name' request/response */
+/**
+ * QmiNasPlmnEncodingScheme:
+ * @QMI_NAS_PLMN_ENCODING_SCHEME_GSM: GSM default alphabet packed encoding (ETSI GSM 03.38)
+ * @QMI_NAS_PLMN_ENCODING_SCHEME_UCS2LE: UCS-2 little-endian
+ *
+ * PLMN name encoding schemes.  See 3GPP TS 24.008 section "Network Name
+ * information element".
+ *
+ * Since: 1.18
+ */
+typedef enum {
+    QMI_NAS_PLMN_ENCODING_SCHEME_GSM    = 0,
+    QMI_NAS_PLMN_ENCODING_SCHEME_UCS2LE = 1,
+} QmiNasPlmnEncodingScheme;
+
+/**
+ * QmiNasNetworkNameDisplayCondition:
+ * @QMI_NAS_NETWORK_NAME_DISPLAY_CONDITION_DISPLAY_REGISTERED_PLMN_IF_KNOWN_NETWORK: if
+ * set, display of the registered PLMN is required when the registered PLMN is either
+ * the HPLMN or a PLMN in the Service Provider PLMN List (see EFspdi). Otherwise
+ * display of the registered PLMN is not required in this case.
+ * @QMI_NAS_NETWORK_NAME_DISPLAY_CONDITION_DISPLAY_SPN_NOT_REQUIRED_IF_UNKNOWN_NETWORK: if
+ * set, display of the Service Provider Name is not required when registered PLMN is
+ * neither HPLMN nor a PLMN in the service provider PLMN list (see EFspdi). If not set,
+ * SPN display is required in this case.
+ *
+ * Flags used to control display of the PLMN name and Service Provider Name. See
+ * 3GPP TS 51.011 descripton of the EFspn SIM file for more details.
+ *
+ * Since: 1.18
+ */
+typedef enum {
+    QMI_NAS_NETWORK_NAME_DISPLAY_CONDITION_DISPLAY_REGISTERED_PLMN_IF_KNOWN_NETWORK    = 1 << 0,
+    QMI_NAS_NETWORK_NAME_DISPLAY_CONDITION_DISPLAY_SPN_NOT_REQUIRED_IF_UNKNOWN_NETWORK = 1 << 1,
+} QmiNasNetworkNameDisplayCondition;
+
+/**
+ * QmiNasPlmnNameCountryInitials:
+ * @QMI_NAS_PLMN_NAME_COUNTRY_INITIALS_DO_NOT_ADD: don't add country initials
+ * @QMI_NAS_PLMN_NAME_COUNTRY_INIITALS_ADD: add country initials
+ * @QMI_NAS_PLMN_NAME_COUNTRY_INIITALS_UNSPECIFIED: unspecified
+ *
+ * PLMN name country initials options. See 3GPP TS 24.008
+ * section "Network Name information element".
+ *
+ * Since: 1.18
+ */
+typedef enum {
+    QMI_NAS_PLMN_NAME_COUNTRY_INITIALS_DO_NOT_ADD  = 0,
+    QMI_NAS_PLMN_NAME_COUNTRY_INIITALS_ADD         = 1,
+    QMI_NAS_PLMN_NAME_COUNTRY_INIITALS_UNSPECIFIED = 0xFF,
+} QmiNasPlmnNameCountryInitials;
+
+/**
+ * QmiNasPlmnNameSpareBits:
+ * @QMI_NAS_PLMN_NAME_SPARE_BITS_UNKNOWN: unknown
+ * @QMI_NAS_PLMN_NAME_SPARE_BITS_BIT_8: bit 8 is spare
+ * @QMI_NAS_PLMN_NAME_SPARE_BITS_BITS_78: bits 7 - 8 are spare
+ * @QMI_NAS_PLMN_NAME_SPARE_BITS_BITS_68: bits 6 - 8 are spare
+ * @QMI_NAS_PLMN_NAME_SPARE_BITS_BITS_58: bits 5 - 8 are spare
+ * @QMI_NAS_PLMN_NAME_SPARE_BITS_BITS_48: bits 4 - 8 are spare
+ * @QMI_NAS_PLMN_NAME_SPARE_BITS_BITS_38: bits 3 - 8 are spare
+ * @QMI_NAS_PLMN_NAME_SPARE_BITS_BITS_28: bits 2 - 8 are spare
+ *
+ * PLMN name spare bits in last octet of a network name.  See 3GPP TS 24.008
+ * section "Network Name information element".
+ *
+ * Since: 1.18
+ */
+typedef enum {
+    QMI_NAS_PLMN_NAME_SPARE_BITS_UNKNOWN = 0,
+    QMI_NAS_PLMN_NAME_SPARE_BITS_BIT_8   = 1,
+    QMI_NAS_PLMN_NAME_SPARE_BITS_BITS_78 = 2,
+    QMI_NAS_PLMN_NAME_SPARE_BITS_BITS_68 = 3,
+    QMI_NAS_PLMN_NAME_SPARE_BITS_BITS_58 = 4,
+    QMI_NAS_PLMN_NAME_SPARE_BITS_BITS_48 = 5,
+    QMI_NAS_PLMN_NAME_SPARE_BITS_BITS_38 = 6,
+    QMI_NAS_PLMN_NAME_SPARE_BITS_BITS_28 = 7,
+} QmiNasPlmnNameSpareBits;
+
 #endif /* _LIBQMI_GLIB_QMI_ENUMS_NAS_H_ */
