@@ -18,11 +18,14 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2012-2015 Aleksander Morgado <aleksander@aleksander.es>
  * Copyright (C) 2012-2015 Dan Williams <dcbw@redhat.com>
+ * Copyright (C) 2012-2017 Aleksander Morgado <aleksander@aleksander.es>
  */
 
 #include <config.h>
+
+#define _GNU_SOURCE
+#include <stdlib.h>
 #include <string.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -140,20 +143,6 @@ print_read_bytes_trace (const gchar *type,
 #define print_read_bytes_trace(...)
 #endif
 
-/**
- * qmi_utils_read_guint8_from_buffer:
- * @buffer: a buffer with raw binary data.
- * @buffer_size: size of @buffer.
- * @out: return location for the read variable.
- *
- * Reads an unsigned byte from the buffer.
- *
- * The user needs to make sure that at least 1 byte is available
- * in the buffer.
- *
- * Also note that both @buffer and @buffer_size get updated after the 1 byte
- * read.
- */
 void
 qmi_utils_read_guint8_from_buffer (const guint8 **buffer,
                                    guint16       *buffer_size,
@@ -172,20 +161,6 @@ qmi_utils_read_guint8_from_buffer (const guint8 **buffer,
     *buffer_size = (*buffer_size) - 1;
 }
 
-/**
- * qmi_utils_read_gint8_from_buffer:
- * @buffer: a buffer with raw binary data.
- * @buffer_size: size of @buffer.
- * @out: return location for the read variable.
- *
- * Reads a signed byte from the buffer.
- *
- * The user needs to make sure that at least 1 byte is available
- * in the buffer.
- *
- * Also note that both @buffer and @buffer_size get updated after the 1 byte
- * read.
- */
 void
 qmi_utils_read_gint8_from_buffer (const guint8 **buffer,
                                   guint16       *buffer_size,
@@ -204,23 +179,6 @@ qmi_utils_read_gint8_from_buffer (const guint8 **buffer,
     *buffer_size = (*buffer_size) - 1;
 }
 
-/**
- * qmi_utils_read_guint16_from_buffer:
- * @buffer: a buffer with raw binary data.
- * @buffer_size: size of @buffer.
- * @endian: endianness of firmware value; swapped to host byte order if necessary
- * @out: return location for the read variable.
- *
- * Reads an unsigned 16-bit integer from the buffer. The number in the buffer is
- * expected to be given in the byte order specificed by @endian, and this method
- * takes care of converting the read value to the proper host endianness.
- *
- * The user needs to make sure that at least 2 bytes are available
- * in the buffer.
- *
- * Also note that both @buffer and @buffer_size get updated after the 2 bytes
- * read.
- */
 void
 qmi_utils_read_guint16_from_buffer (const guint8 **buffer,
                                     guint16       *buffer_size,
@@ -244,23 +202,6 @@ qmi_utils_read_guint16_from_buffer (const guint8 **buffer,
     *buffer_size = (*buffer_size) - 2;
 }
 
-/**
- * qmi_utils_read_gint16_from_buffer:
- * @buffer: a buffer with raw binary data.
- * @buffer_size: size of @buffer.
- * @endian: endianness of firmware value; swapped to host byte order if necessary
- * @out: return location for the read variable.
- *
- * Reads a signed 16-bit integer from the buffer. The number in the buffer is
- * expected to be given in the byte order specified by @endian, and this method
- * takes care of converting the read value to the proper host endianness.
- *
- * The user needs to make sure that at least 2 bytes are available
- * in the buffer.
- *
- * Also note that both @buffer and @buffer_size get updated after the 2 bytes
- * read.
- */
 void
 qmi_utils_read_gint16_from_buffer (const guint8 **buffer,
                                    guint16       *buffer_size,
@@ -284,23 +225,6 @@ qmi_utils_read_gint16_from_buffer (const guint8 **buffer,
     *buffer_size = (*buffer_size) - 2;
 }
 
-/**
- * qmi_utils_read_guint32_from_buffer:
- * @buffer: a buffer with raw binary data.
- * @buffer_size: size of @buffer.
- * @endian: endianness of firmware value; swapped to host byte order if necessary
- * @out: return location for the read variable.
- *
- * Reads an unsigned 32-bit integer from the buffer. The number in the buffer is
- * expected to be given in the byte order specified by @endian, and this method
- * takes care of converting the read value to the proper host endianness.
- *
- * The user needs to make sure that at least 4 bytes are available
- * in the buffer.
- *
- * Also note that both @buffer and @buffer_size get updated after the 4 bytes
- * read.
- */
 void
 qmi_utils_read_guint32_from_buffer (const guint8 **buffer,
                                     guint16       *buffer_size,
@@ -324,23 +248,6 @@ qmi_utils_read_guint32_from_buffer (const guint8 **buffer,
     *buffer_size = (*buffer_size) - 4;
 }
 
-/**
- * qmi_utils_read_gint32_from_buffer:
- * @buffer: a buffer with raw binary data.
- * @buffer_size: size of @buffer.
- * @endian: endianness of firmware value; swapped to host byte order if necessary
- * @out: return location for the read variable.
- *
- * Reads a signed 32-bit integer from the buffer. The number in the buffer is
- * expected to be given in the byte order specified by @endian, and this method
- * takes care of converting the read value to the proper host endianness.
- *
- * The user needs to make sure that at least 4 bytes are available
- * in the buffer.
- *
- * Also note that both @buffer and @buffer_size get updated after the 4 bytes
- * read.
- */
 void
 qmi_utils_read_gint32_from_buffer (const guint8 **buffer,
                                    guint16       *buffer_size,
@@ -364,23 +271,6 @@ qmi_utils_read_gint32_from_buffer (const guint8 **buffer,
     *buffer_size = (*buffer_size) - 4;
 }
 
-/**
- * qmi_utils_read_guint64_from_buffer:
- * @buffer: a buffer with raw binary data.
- * @buffer_size: size of @buffer.
- * @endian: endianness of firmware value; swapped to host byte order if necessary
- * @out: return location for the read variable.
- *
- * Reads an unsigned 64-bit integer from the buffer. The number in the buffer is
- * expected to be given in the byte order specified by @endian, and this method
- * takes care of converting the read value to the proper host endianness.
- *
- * The user needs to make sure that at least 8 bytes are available
- * in the buffer.
- *
- * Also note that both @buffer and @buffer_size get updated after the 8 bytes
- * read.
- */
 void
 qmi_utils_read_guint64_from_buffer (const guint8 **buffer,
                                     guint16       *buffer_size,
@@ -404,23 +294,6 @@ qmi_utils_read_guint64_from_buffer (const guint8 **buffer,
     *buffer_size = (*buffer_size) - 8;
 }
 
-/**
- * qmi_utils_read_gint64_from_buffer:
- * @buffer: a buffer with raw binary data.
- * @buffer_size: size of @buffer.
- * @endian: endianness of firmware value; swapped to host byte order if necessary
- * @out: return location for the read variable.
- *
- * Reads a signed 64-bit integer from the buffer. The number in the buffer is
- * expected to be given in the byte order specified by @endian, and this method
- * takes care of converting the read value to the proper host endianness.
- *
- * The user needs to make sure that at least 8 bytes are available
- * in the buffer.
- *
- * Also note that both @buffer and @buffer_size get updated after the 8 bytes
- * read.
- */
 void
 qmi_utils_read_gint64_from_buffer (const guint8 **buffer,
                                    guint16       *buffer_size,
@@ -444,25 +317,6 @@ qmi_utils_read_gint64_from_buffer (const guint8 **buffer,
     *buffer_size = (*buffer_size) - 8;
 }
 
-/**
- * qmi_utils_read_sized_guint_from_buffer:
- * @buffer: a buffer with raw binary data.
- * @buffer_size: size of @buffer.
- * @n_bytes: number of bytes to read.
- * @endian: endianness of firmware value; swapped to host byte order if necessary
- * @out: return location for the read variable.
- *
- * Reads a @n_bytes-sized unsigned integer from the buffer. The number in the
- * buffer is expected to be given in the byte order specified by @endian, and
- * this method takes care of converting the read value to the proper host
- * endianness.
- *
- * The user needs to make sure that at least @n_bytes bytes are available
- * in the buffer.
- *
- * Also note that both @buffer and @buffer_size get updated after the @n_bytes
- * bytes read.
- */
 void
 qmi_utils_read_sized_guint_from_buffer (const guint8 **buffer,
                                         guint16       *buffer_size,
@@ -497,20 +351,6 @@ qmi_utils_read_sized_guint_from_buffer (const guint8 **buffer,
     *buffer_size = (*buffer_size) - n_bytes;
 }
 
-/**
- * qmi_utils_read_gfloat_from_buffer:
- * @buffer: a buffer with raw binary data.
- * @buffer_size: size of @buffer.
- * @out: return location for the read variable.
- *
- * Reads a 32-bit floating-point number from the buffer.
- *
- * The user needs to make sure that at least 4 bytes are available
- * in the buffer.
- *
- * Also note that both @buffer and @buffer_size get updated after the 4 bytes
- * read.
- */
 void
 qmi_utils_read_gfloat_from_buffer (const guint8 **buffer,
                                    guint16       *buffer_size,
@@ -530,19 +370,6 @@ qmi_utils_read_gfloat_from_buffer (const guint8 **buffer,
     *buffer_size = (*buffer_size) - 4;
 }
 
-/**
- * qmi_utils_write_guint8_to_buffer:
- * @buffer: a buffer.
- * @buffer_size: size of @buffer.
- * @in: location of the variable to be written.
- *
- * Writes an unsigned byte into the buffer.
- *
- * The user needs to make sure that the buffer is at least 1 byte long.
- *
- * Also note that both @buffer and @buffer_size get updated after the 1 byte
- * write.
- */
 void
 qmi_utils_write_guint8_to_buffer (guint8  **buffer,
                                   guint16  *buffer_size,
@@ -559,19 +386,6 @@ qmi_utils_write_guint8_to_buffer (guint8  **buffer,
     *buffer_size = (*buffer_size) - 1;
 }
 
-/**
- * qmi_utils_write_gint8_to_buffer:
- * @buffer: a buffer.
- * @buffer_size: size of @buffer.
- * @in: location of the variable to be written.
- *
- * Writes a signed byte into the buffer.
- *
- * The user needs to make sure that the buffer is at least 1 byte long.
- *
- * Also note that both @buffer and @buffer_size get updated after the 1 byte
- * write.
- */
 void
 qmi_utils_write_gint8_to_buffer (guint8  **buffer,
                                  guint16  *buffer_size,
@@ -588,22 +402,6 @@ qmi_utils_write_gint8_to_buffer (guint8  **buffer,
     *buffer_size = (*buffer_size) - 1;
 }
 
-/**
- * qmi_utils_write_guint16_to_buffer:
- * @buffer: a buffer.
- * @buffer_size: size of @buffer.
- * @endian: endianness of firmware value; swapped from host byte order if necessary
- * @in: location of the variable to be written.
- *
- * Writes an unsigned 16-bit integer into the buffer. The number to be written
- * is expected to be given in host endianness, and this method takes care of
- * converting the value written to the byte order specified by @endian.
- *
- * The user needs to make sure that the buffer is at least 2 bytes long.
- *
- * Also note that both @buffer and @buffer_size get updated after the 2 bytes
- * write.
- */
 void
 qmi_utils_write_guint16_to_buffer (guint8  **buffer,
                                    guint16  *buffer_size,
@@ -627,22 +425,6 @@ qmi_utils_write_guint16_to_buffer (guint8  **buffer,
     *buffer_size = (*buffer_size) - 2;
 }
 
-/**
- * qmi_utils_write_gint16_to_buffer:
- * @buffer: a buffer.
- * @buffer_size: size of @buffer.
- * @endian: endianness of firmware value; swapped from host byte order if necessary
- * @in: location of the variable to be written.
- *
- * Writes a signed 16-bit integer into the buffer. The number to be written
- * is expected to be given in host endianness, and this method takes care of
- * converting the value written to the byte order specified by @endian.
- *
- * The user needs to make sure that the buffer is at least 2 bytes long.
- *
- * Also note that both @buffer and @buffer_size get updated after the 2 bytes
- * write.
- */
 void
 qmi_utils_write_gint16_to_buffer (guint8  **buffer,
                                   guint16  *buffer_size,
@@ -666,22 +448,6 @@ qmi_utils_write_gint16_to_buffer (guint8  **buffer,
     *buffer_size = (*buffer_size) - 2;
 }
 
-/**
- * qmi_utils_write_guint32_to_buffer:
- * @buffer: a buffer.
- * @buffer_size: size of @buffer.
- * @endian: endianness of firmware value; swapped from host byte order if necessary
- * @in: location of the variable to be written.
- *
- * Writes an unsigned 32-bit integer into the buffer. The number to be written
- * is expected to be given in host endianness, and this method takes care of
- * converting the value written to the byte order specified by @endian.
- *
- * The user needs to make sure that the buffer is at least 4 bytes long.
- *
- * Also note that both @buffer and @buffer_size get updated after the 4 bytes
- * write.
- */
 void
 qmi_utils_write_guint32_to_buffer (guint8  **buffer,
                                    guint16  *buffer_size,
@@ -705,22 +471,6 @@ qmi_utils_write_guint32_to_buffer (guint8  **buffer,
     *buffer_size = (*buffer_size) - 4;
 }
 
-/**
- * qmi_utils_write_gint32_to_buffer:
- * @buffer: a buffer.
- * @buffer_size: size of @buffer.
- * @endian: endianness of firmware value; swapped from host byte order if necessary
- * @in: location of the variable to be written.
- *
- * Writes a signed 32-bit integer into the buffer. The number to be written
- * is expected to be given in host endianness, and this method takes care of
- * converting the value written to the byte order specified by @endian.
- *
- * The user needs to make sure that the buffer is at least 4 bytes long.
- *
- * Also note that both @buffer and @buffer_size get updated after the 4 bytes
- * write.
- */
 void
 qmi_utils_write_gint32_to_buffer (guint8  **buffer,
                                   guint16  *buffer_size,
@@ -744,22 +494,6 @@ qmi_utils_write_gint32_to_buffer (guint8  **buffer,
     *buffer_size = (*buffer_size) - 4;
 }
 
-/**
- * qmi_utils_write_guint64_to_buffer:
- * @buffer: a buffer.
- * @buffer_size: size of @buffer.
- * @endian: endianness of firmware value; swapped from host byte order if necessary
- * @in: location of the variable to be written.
- *
- * Writes an unsigned 64-bit integer into the buffer. The number to be written
- * is expected to be given in host endianness, and this method takes care of
- * converting the value written to the byte order specified by @endian.
- *
- * The user needs to make sure that the buffer is at least 8 bytes long.
- *
- * Also note that both @buffer and @buffer_size get updated after the 8 bytes
- * write.
- */
 void
 qmi_utils_write_guint64_to_buffer (guint8  **buffer,
                                    guint16  *buffer_size,
@@ -783,22 +517,6 @@ qmi_utils_write_guint64_to_buffer (guint8  **buffer,
     *buffer_size = (*buffer_size) - 8;
 }
 
-/**
- * qmi_utils_write_gint64_to_buffer:
- * @buffer: a buffer.
- * @buffer_size: size of @buffer.
- * @endian: endianness of firmware value; swapped from host byte order if necessary
- * @in: location of the variable to be written.
- *
- * Writes a signed 64-bit integer into the buffer. The number to be written
- * is expected to be given in host endianness, and this method takes care of
- * converting the value written to the byte order specified by @endian.
- *
- * The user needs to make sure that the buffer is at least 8 bytes long.
- *
- * Also note that both @buffer and @buffer_size get updated after the 8 bytes
- * write.
- */
 void
 qmi_utils_write_gint64_to_buffer (guint8  **buffer,
                                   guint16  *buffer_size,
@@ -822,23 +540,6 @@ qmi_utils_write_gint64_to_buffer (guint8  **buffer,
     *buffer_size = (*buffer_size) - 8;
 }
 
-/**
- * qmi_utils_write_sized_guint_to_buffer:
- * @buffer: a buffer.
- * @buffer_size: size of @buffer.
- * @n_bytes: number of bytes to write.
- * @endian: endianness of firmware value; swapped from host byte order if necessary
- * @in: location of the variable to be written.
- *
- * Writes a @n_bytes-sized unsigned integer into the buffer. The number to be
- * written is expected to be given in host endianness, and this method takes
- * care of converting the value written to the byte order specified by @endian.
- *
- * The user needs to make sure that the buffer is at least @n_bytes bytes long.
- *
- * Also note that both @buffer and @buffer_size get updated after the @n_bytes
- * bytes write.
- */
 void
 qmi_utils_write_sized_guint_to_buffer (guint8  **buffer,
                                        guint16  *buffer_size,
@@ -875,21 +576,6 @@ qmi_utils_write_sized_guint_to_buffer (guint8  **buffer,
     *buffer_size = (*buffer_size) - n_bytes;
 }
 
-/**
- * qmi_utils_read_string_from_buffer:
- * @buffer: a buffer with raw binary data.
- * @buffer_size: size of @buffer.
- * @length_prefix_size: size of the length prefix integer in bits.
- * @max_size: maximum number of bytes to read, or 0 to read all available bytes.
- * @out: return location for the read string. The returned value should be freed with g_free().
- *
- * Reads a string from the buffer.
- *
- * If @length_prefix_size is greater than 0, only the amount of bytes given
- * there will be read. Otherwise, up to @buffer_size bytes will be read.
- *
- * Also note that both @buffer and @buffer_size get updated after the write.
- */
 void
 qmi_utils_read_string_from_buffer (const guint8 **buffer,
                                    guint16       *buffer_size,
@@ -946,18 +632,6 @@ qmi_utils_read_string_from_buffer (const guint8 **buffer,
     *buffer_size = (*buffer_size) - string_length;
 }
 
-/**
- * qmi_utils_read_fixed_size_string_from_buffer:
- * @buffer: a buffer with raw binary data.
- * @buffer_size: size of @buffer.
- * @fixed_size: number of bytes to read.
- * @out: buffer preallocated by the client, with at least @fixed_size bytes.
- *
- * Reads a @fixed_size-sized string from the buffer into the @out buffer.
- *
- * Also note that both @buffer and @buffer_size get updated after the
- * @fixed_size bytes read.
- */
 void
 qmi_utils_read_fixed_size_string_from_buffer (const guint8 **buffer,
                                               guint16       *buffer_size,
@@ -975,23 +649,6 @@ qmi_utils_read_fixed_size_string_from_buffer (const guint8 **buffer,
     *buffer_size = (*buffer_size) - fixed_size;
 }
 
-/**
- * qmi_utils_write_string_to_buffer:
- * @buffer: a buffer.
- * @buffer_size: size of @buffer.
- * @length_prefix_size: size of the length prefix integer in bits.
- * @in: string to write.
- *
- * Writes a string to the buffer.
- *
- * If @length_prefix_size is greater than 0, a length prefix integer will be
- * included in the write operation.
- *
- * The user needs to make sure that the buffer has enough space for both the
- * whole string and the length prefix.
- *
- * Also note that both @buffer and @buffer_size get updated after the write.
- */
 void
 qmi_utils_write_string_to_buffer (guint8      **buffer,
                                   guint16      *buffer_size,
@@ -1044,21 +701,6 @@ qmi_utils_write_string_to_buffer (guint8      **buffer,
     *buffer_size = (*buffer_size) - len;
 }
 
-/**
- * qmi_utils_write_fixed_size_string_to_buffer:
- * @buffer: a buffer.
- * @buffer_size: size of @buffer.
- * @fixed_size: number of bytes to write.
- * @in: string to write.
- *
- * Writes a @fixed_size-sized string to the buffer, without any length prefix.
- *
- * The user needs to make sure that the buffer is at least @fixed_size bytes
- * long.
- *
- * Also note that both @buffer and @buffer_size get updated after the
- * @fixed_size bytes write.
- */
 void
 qmi_utils_write_fixed_size_string_to_buffer (guint8      **buffer,
                                              guint16      *buffer_size,
@@ -1078,27 +720,49 @@ qmi_utils_write_fixed_size_string_to_buffer (guint8      **buffer,
 
 /*****************************************************************************/
 
+gchar *
+__qmi_utils_get_driver (const gchar *cdc_wdm_path)
+{
+    static const gchar *subsystems[] = { "usbmisc", "usb" };
+    guint i;
+    gchar *device_basename;
+    gchar *driver = NULL;
+
+    device_basename = g_path_get_basename (cdc_wdm_path);
+
+    for (i = 0; !driver && i < G_N_ELEMENTS (subsystems); i++) {
+        gchar *tmp;
+        gchar *path;
+
+        /* driver sysfs can be built directly using subsystem and name; e.g. for subsystem
+         * usbmisc and name cdc-wdm0:
+         *    $ realpath /sys/class/usbmisc/cdc-wdm0/device/driver
+         *    /sys/bus/usb/drivers/qmi_wwan
+         */
+        tmp = g_strdup_printf ("/sys/class/%s/%s/device/driver", subsystems[i], device_basename);
+        path = canonicalize_file_name (tmp);
+        g_free (tmp);
+
+        if (g_file_test (path, G_FILE_TEST_EXISTS))
+            driver = g_path_get_basename (path);
+        g_free (path);
+    }
+
+    g_free (device_basename);
+
+    return driver;
+}
+
+/*****************************************************************************/
+
 static volatile gint __traces_enabled = FALSE;
 
-/**
- * qmi_utils_get_traces_enabled:
- *
- * Checks whether QMI message traces are currently enabled.
- *
- * Returns: %TRUE if traces are enabled, %FALSE otherwise.
- */
 gboolean
 qmi_utils_get_traces_enabled (void)
 {
     return (gboolean) g_atomic_int_get (&__traces_enabled);
 }
 
-/**
- * qmi_utils_set_traces_enabled:
- * @enabled: %TRUE to enable traces, %FALSE to disable them.
- *
- * Sets whether QMI message traces are enabled or disabled.
- */
 void
 qmi_utils_set_traces_enabled (gboolean enabled)
 {
